/*======================================================================================
t@CF	LCD.c
@\TvF		LCDC^[tF[X[`
======================================================================================*/

#include "lcd.h"
#include "timera.h"
#include "iodefine.h"

#define CMD 0
#define DAT 1
#define WRITEACCESS 0
#define READACCESS 1
#define	LCDREAD		0x00			/*|[gSrbg*/
#define	LCDWRITE	0xff			/*|[gSrbgo*/
#define EBIT	IO.PDR1.BIT.B2		/*LCD̃Cl[urbg*/
#define RWBIT	IO.PDR1.BIT.B1		/*LCD̃ANZXwrbg*/
#define RSBIT	IO.PDR1.BIT.B0		/*LCD̃WX^IrbgBLCDRAMɃANZX鎞ɃZbg*/
#define	LCDPORT	IO.PDR5.BYTE		/*LCD̃f[^|[g*/
#define LCDDIRECTION	IO.PCR5		/*LCDɐڑĂ|[gT̕WX^*/

#ifndef bool 
#define	bool char
#endif

#ifndef true
#define true 1
#endif

#ifndef false
#define false 0
#endif

static bool Is4BitMode=false;

#define LCDWaitReady() 		while(IsLCDBusy())
#define lcdoutxy(x,y,s) LcdXy(x,y); LcdPuts(s)

/*======================================================================================
֐F		CmWrite
@\F			LCDɈŗ^f[^
======================================================================================*/
void CmWrite(char cmd)
{
	LCDDIRECTION=LCDWRITE;
	RWBIT=WRITEACCESS;
	if(Is4BitMode)
		LCDPORT=(LCDPORT&0x0F)|(cmd&0xF0);
	else
		LCDPORT=cmd;
	mcsdelay(20);
	EBIT=1;
	mcsdelay(20);
	EBIT=0;
	mcsdelay(20);
}

/*======================================================================================
֐F		CmRead
@\F			LCD̃|[gf[^ǂݍ
======================================================================================*/
char CmRead()
{
	char dat;

	LCDDIRECTION=LCDREAD;
	RWBIT=READACCESS;
	mcsdelay(20);
	EBIT=1;
	mcsdelay(20);
	dat=LCDPORT;
	if(Is4BitMode) dat&=0xF0;
	EBIT=0;
	return dat;
}

/*======================================================================================
֐F		IsLCDBusy
@\F			LCDBUSYԂ`FbNBLCD쒆̎ɂtrueԂB
======================================================================================*/
bool IsLCDBusy()
{
	char reg;
	RSBIT=CMD;
	RWBIT=READACCESS;
	msdelay(1);
	reg=CmRead();
	if(Is4BitMode) reg|=CmRead()>>4;
	msdelay(1);
	if(reg & 0x80)
		return true;
	return false;
}

/*======================================================================================
֐F		LcdCmd
@\F			LCDɃR}h𑗂B
======================================================================================*/
void LcdCmd(int cmd)
{
	RSBIT=CMD;
	CmWrite(cmd);
	if(Is4BitMode)
		CmWrite(cmd<<4);
}


/*======================================================================================
֐F		LcdDat
@\F			LCDRAMɏށB
======================================================================================*/
void LcdDat(int dat)
{
	RSBIT=DAT;
	CmWrite(dat);
	if(Is4BitMode)
		CmWrite(dat<<4);
}

/*======================================================================================
֐F		LcdCls
@\F			LCD̕\NAB
======================================================================================*/
void LcdCls()
{
	RSBIT=CMD;
	LcdCmd(1);
	msdelay(2);
}

/*======================================================================================
֐F		LcdDisplayMode
@\F			LCD̕\[hݒ肷
				disp:		LCDŜ̕\ON/OFF
				cursor:		J[\\ON/OFF
				blink		J[\ʒũ̕uNON/OFF
======================================================================================*/
void LcdDisplayMode(char disp, char cursor,char blink)
{
	char mode=0x08;
	if(disp) mode|=0x04;
	if(cursor) mode|=0x02;
	if(blink) mode|=0x01;
	RSBIT=CMD;
	LcdCmd(mode);
	mcsdelay(40);
}

/*======================================================================================
֐F		LcdInit
@\F			LCD̏BN1񂾂Ăяo
				mode==4̎A4bitANZXBȊO8bitANZX
======================================================================================*/
void LcdInit(char mode)
{
	//|[g̏
 	IO.PCR1=0x07;	//P10-12 OUTPUT
 	IO.PDR1.BYTE=0;		//EBIT=RWBIT=RSBIT=0
	IO.PMR5.BYTE=0;
	LCDDIRECTION=LCDWRITE;
	LCDPORT=0;		//0ŏ

	RSBIT=CMD;
	msdelay(15);
	CmWrite(0x30);		//8bitMode Set
	msdelay(5);
	CmWrite(0x30);
	mcsdelay(100);
	CmWrite(0x30);
	mcsdelay(100);

	if(mode==4){
		Is4BitMode=true;
		CmWrite(0x20);
		mcsdelay(40);
		LcdCmd(0x28);	//4BitMode Set,5*7font,1/16Duty
	}else{
		Is4BitMode=false;
		CmWrite(0x38);	//8BitMode Set,5*7font,1/16Duty
		mcsdelay(40);
	}
	
	//琳K̃ANZXɂȂ
	LcdDisplayMode(false,false,false);
	LcdCls();
	LcdCmd(0x06);	//EntryMode SetB񂾍ۂɃAhXCNgAJ[\Eړ
	mcsdelay(40);
	LcdDisplayMode(true,true,true);
}

/*======================================================================================
֐F		LcdPutc
@\F			ݐݒ肳ĂLCDDDRAMAhX1o͂
======================================================================================*/
void LcdPutc(char c)
{
	LcdDat(c);
}

/*======================================================================================
֐F		LcdPuts
@\F			ݐݒ肳ĂLCDDDRAMAhX֕o͂
======================================================================================*/
void LcdPuts(char *str)
{
	while(*str){
		LcdPutc(*str);
		str++;
	}
}

/*======================================================================================
֐F		LcdInit
@\F			DDRAM̃AhXZbg
				x:i0`7j
				Y:si0`1j
======================================================================================*/
void LcdXy(int x,int y)
{
	unsigned char adr;
	adr=(x+(y%2)*0x40) | 0x80;
	LcdCmd(adr);
	msdelay(10);
}

